/*
@@@BUILDINFO@@@ Adobe Watermark.jsx 2.0.5  23-Apr-2010
*/
﻿
/**************************************************************************
	ADOBE SYSTEMS INCORPORATED
	 Copyright 2010 Adobe Systems Incorporated
	 All Rights Reserved.

	NOTICE:  Adobe permits you to use, modify, and distribute this file
	in accordance with the terms of the Adobe license agreement accompanying
	it.  If you have received this file from a source other than Adobe, then
	your use, modification, or distribution of it requires the prior written
	permission of Adobe.
**************************************************************************/

// i didn't indent cuz that would be annoying
try {
	

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// // Some globals at the top for easy editing
var watermark = new Object;
watermark.showAlerts = true; // use this for user alerts
watermark.debugAlerts = false; // use this for debug purposes only, should be false
watermark.leaveWatermarkOpenWhenProcessingOpenImages = false;
watermark.tempDocName = localize("$$$/watermark/string/tempdocname=Preview Watermark Creator");

///////////////////////////////////////////////////////////////////////////////
// Function: runProcess
// Usage: Main processing routine called from the Watermark panel
// Input: string of the object from the panel, must use eval on the parameters
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////
watermark.runProcess = function( inParamsAsSource ) {

	var oldPref = app.preferences.rulerUnits;
		
	try {

	    app.preferences.rulerUnits = Units.PIXELS;
		
		var params = watermark.paramsToParams( inParamsAsSource );

		if ( params.cbUseOpenImages && app.documents.length == 0 ) {
			var e = new Error();
			e.description = localize("$$$/watermark/alert/noopenimage=No open images.");
			throw( e );
		}
		
		var f = Folder( params.stSelectedOutput );
		if ( ! params.cbOnOneImage && ! f.exists ) {
			var e = new Error();
			e.description = localize("$$$/watermark/alert/nodestinationfolder=Destination folder does not exist.");
			throw( e );
		}
	
		if ( params.rbUseLogoFile ) {
			var f = File( params.stSelectedWatermark );
			if ( ! f.exists) {
				var e = new Error();
				e.description = localize("$$$/watermark/alert/nowatermarkfile=Watermark logo file was not selected or does not exist.");
				throw( e );
			}
		}
		
		for ( var i = app.documents.length - 1; i >= 0; i-- ) {
			if ( app.documents[i].name == watermark.tempDocName ) {
				app.documents[i].close( SaveOptions.DONOTSAVECHANGES );	
			}	
		}
		
		var inputFiles;
		if ( params.cbUseOpenImages ) {
			inputFiles = watermark.openDocs();
		} else {
			var f = Folder( params.stSelectedInput );
			if ( ! f.exists ) {
				var e = new Error();
				e.description = localize("$$$/watermark/alert/noinputfolder=Input folder does not exist.");
				throw( e );
			} else {
				inputFiles = f.getFiles();
				// A list of file extensions and types PS can read
				gFileExtensionsToRead = app.windowsFileTypes;
				gFileTypesToRead = app.macintoshFileTypes;
			}
		}

		if ( app.documents.length ) {
			var originalDocument = app.activeDocument;
		}
		
		var processedOneFile = false;
		
		for ( var i = 0; i < inputFiles.length; i++ ) {
	
			var alreadyOpenFile;
			
			if ( params.cbUseOpenImages ) {
				if ( ! params.cbOnOneImage ) {
					activeDocument = inputFiles[i];
				}
				var doc = activeDocument;
				try {
					var fullPath = doc.fullName.fsName;
				} catch(e) {
					var fullPath = "Watermark";
				} 
			} else {
				if ( inputFiles[i] instanceof File &&
				     ! inputFiles[i].hidden &&
				     ( watermark.isFileOneOfThese( inputFiles[i], gFileExtensionsToRead ) ||
				       watermark.isFileOneOfTheseTypes( inputFiles[i], gFileTypesToRead ) )
				      ) {
					alreadyOpenFile = undefined;
					for ( var s = 0; s < app.documents.length; s++ ) {
						try {
							if ( inputFiles[i].toString() == app.documents[s].fullName.toString() ) {
								alreadyOpenFile = app.documents[s];
								break;
							}
						} catch(e) { ; } // just go to the next file
					}
					if ( typeof(alreadyOpenFile) != "undefined" && alreadyOpenFile instanceof Document ) {
						doc = alreadyOpenFile;
					} else {
						doc = app.open( inputFiles[i] );
					}
					var docOpened = doc;
					var fullPath = inputFiles[i];
				} else {
					continue;
				}		
			}
			
			doc = activeDocument.duplicate( watermark.tempDocName );
			
			watermark.convertModeIfNeeded( doc, undefined ); // pick the default mode if needed
 
 			if ( params.rbSaveForWeb ) {
				var r = 72;
			} else {
				var r = Number( params.etResolution );
			}
			if ( ! isNaN( r ) ) {
				doc.resizeImage( undefined, undefined, r, ResampleMethod.NONE );
			}

			if ( params.cbSizeToFit ) {
				var unitWidth = watermark.stringToUnitValue( params.etSizeToFitW, "px" );
				var unitHeight = watermark.stringToUnitValue( params.etSizeToFitH, "px" );
				var w = watermark.unitsToPixels( unitWidth.value, unitWidth.type, doc.resolution, doc.width.as("px") );
				var h = watermark.unitsToPixels( unitHeight.value, unitHeight.type, doc.resolution, doc.height.as("px") );

				watermark.fitImage( w, h, 'Bicubic' );
			}

			doc.flatten();

			if ( params.cbSmartSharpen ) {
				watermark.webSharpen( doc );
			}
			
			if ( params.rbUseLogoFile )
				watermark.watermarkFileImage( doc, params );
			else
				watermark.watermarkFileText( doc, params, inputFiles[i].name );
			
			doc.flatten();
			
			if ( params.cbOnOneImage ) {
				
				if ( ! params.cbUseOpenImages && ! ( alreadyOpenFile instanceof Document ) ) {
					docOpened.close( SaveOptions.DONOTSAVECHANGES );
				}
				
				watermark.fitOnScreen();
				
				processedOneFile = true;
				
				break;
				
			} else {
				
				watermark.mySave( params, fullPath, doc );
			
				if ( params.cbUseOpenImages && watermark.leaveWatermarkOpenWhenProcessingOpenImages ) {
				
					inputFiles[i].close( SaveOptions.DONOTSAVECHANGES );
				
				} else {
		
					doc.close( SaveOptions.DONOTSAVECHANGES );	
			
					if ( ! params.cbUseOpenImages && ! ( alreadyOpenFile instanceof Document ) ) {
						docOpened.close( SaveOptions.DONOTSAVECHANGES );
					}
				}
			}
			
			processedOneFile = true;
			
		} // end for loop
	
		if ( app.documents.length && originalDocument instanceof Document && ! params.cbOnOneImage && params.cbUseOpenImages ) {
			app.activeDocument = originalDocument;
		}
		
		if (  ! processedOneFile ) {
			var e = new Error();
			e.description = localize("$$$/watermark/alert/nofileprocessed=No files processed.");
			throw( e );
		}
	
	} // main try block

	catch(e) { 
		if (watermark.showAlerts) alert(e +" : " + e.line); 
	}
	
	app.preferences.rulerUnits = oldPref; // restore old prefs
	
}

watermark.paramsToParams = function( inParamsAsSource ) {

	var params = eval( decodeURIComponent( unescape( inParamsAsSource ) ) );
	
	return params;
}

///////////////////////////////////////////////////////////////////////////////
// Function: watermarkFileImage
// Usage: Do the watermarking of the document, including the embossing
//        d is the current target document
//        p is the parameters from the panel
// Return: Current document is watermarked
// NOTE: need to optimize this and close at the very end after all files are done
///////////////////////////////////////////////////////////////////////////////
watermark.watermarkFileImage = function( d, p ) {
	try {
	var wasOpen = false;
	var f = File( p.stSelectedWatermark );
	for ( var i = 0; i < app.documents.length; i++ ) {
		try {
			if ( f.toString() == app.documents[i].fullName.toString() ) {
			  var wmFile = app.documents[i];
			  wasOpen = true;
			  break;
			}
		} catch(e) { ; } // just go to the next file
	}
	if ( ! wasOpen) {
		wmFile = app.open( f );
	} else {
		app.activeDocument = wmFile;
	}
	
	var w = watermark.unitsToPixels( d.width.value, d.width.type, d.resolution, d.width.value );
	var h = watermark.unitsToPixels( d.height.value, d.height.type, d.resolution, d.height.value );
	w = w * p.etWaterMarkSize / 100.0;
	h = h * p.etWaterMarkSize / 100.0;
	var x = Math.max( w, h );
	watermark.fitImage( x, x, 'Bicubic' );
	
	watermark.mergeVisible();
	
	watermark.convertModeIfNeeded(app.activeDocument, d.mode);
	
	watermark.duplicateLayerToDocument( "Watermark", d.name );
	
	if ( ! wasOpen) {
		wmFile.close( SaveOptions.DONOTSAVECHANGES );
	}
	
	app.activeDocument = d;
	
	var pastedLayer = d.activeLayer;
	
	pastedLayer.move(d.artLayers[0], ElementPlacement.PLACEBEFORE);
	
	watermark.alignWatermark( d, p );
	
	d.activeLayer.opacity = p.etWaterMarkOpacity;
	
	} catch(e) { 
		if (watermark.debugAlerts) {
			alert(e + " " + e.line); 
		}
		throw(e); // return false; 
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////
// Function: convertModeIfNeeded
// Usage: If the logo document is not in the correct mode things go bad,
//        check for bad modes and if so switch the mode to the target document
//        w is the current watermark file and the current target document
//        m is the new mode to convert to
// Return: watermark file converted to the target document mode if needed
///////////////////////////////////////////////////////////////////////////////
watermark.convertModeIfNeeded = function(w, m)
{
	if (w.mode == DocumentMode.BITMAP || 
	    w.mode == DocumentMode.INDEXEDCOLOR ||
	    w.mode == DocumentMode.MULTICHANNEL) {
		watermark.conditionalModeChange(m);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: conditionalModeChange
// Usage: convert the active document to the mode
//        m mode of the new document
// Return: activeDocument converted to the new mode via conditional mode change
///////////////////////////////////////////////////////////////////////////////
watermark.conditionalModeChange = function(m) {
	var eventModeChange = stringIDToTypeID( "8cba8cd6-cb66-11d1-bc43-0060b0a13dc4" );
	var descSource = new ActionDescriptor();
	var keySourceMode = charIDToTypeID( "SrcM" );
	var list = new ActionList();
	var keyCondition = charIDToTypeID( "Cndn" );
	var keyBitmap = charIDToTypeID( "UBtm" );
	var keyGrayscale = charIDToTypeID( "UGry" );
	var keyDuotone = charIDToTypeID( "UDtn" );
	var keyIndex = charIDToTypeID( "UInd" );
	var keyRGB = charIDToTypeID( "URGB" );
	var keyCMYK = charIDToTypeID( "UCMY" );
	var keyLab = charIDToTypeID( "ULab" );
	var keyMultichannel = charIDToTypeID( "UMlt" );
	list.putEnumerated( keyCondition, keyBitmap );
	list.putEnumerated( keyCondition, keyGrayscale );
	list.putEnumerated( keyCondition, keyDuotone );
	list.putEnumerated( keyCondition, keyIndex );
	list.putEnumerated( keyCondition, keyRGB );
	list.putEnumerated( keyCondition, keyCMYK );
	list.putEnumerated( keyCondition, keyLab );
	list.putEnumerated( keyCondition, keyMultichannel );
	descSource.putList( keySourceMode, list );
	var keyDestination = charIDToTypeID( "DstM" );
	var descDest = new ActionDescriptor();
	var keyMode = charIDToTypeID( "RGBM" );
	switch (m) {
		case DocumentMode.GRAYSCALE:
			keyMode = charIDToTypeID( 'Grys' );
			break;
		case DocumentMode.DUOTONE:
			keyMode = charIDToTypeID( 'DtnM' );
			break;
		case DocumentMode.CMYK:
			keyMode = charIDToTypeID( 'CMYM' );
			break;
		case DocumentMode.LAB:
			keyMode = charIDToTypeID( 'LbCM' );
			break;
		case DocumentMode.MULTICHANNEL: // keyMode = charIDToTypeID( 'MltC' );
		case DocumentMode.BITMAP: // keyMode =  = charIDToTypeID( 'BtmM' );
		case DocumentMode.INDEXEDCOLOR: //keyMode =  = charIDToTypeID( 'IndC' );
		case DocumentMode.RGB: // already done above
		default: // RGB i guess
			break;
	}
	descSource.putObject( keyDestination, keyMode, descDest );
	executeAction( eventModeChange, descSource, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: watermarkFileText
// Usage: Do the watermarking of the document, specifically for text
//        d is the current target document
//        p is the parameters from the panel
//        docFileName is the name of the original document 
// Return: Current document is watermarked
///////////////////////////////////////////////////////////////////////////////
watermark.watermarkFileText = function( d, p, docFileName ) {

	if ( p.etWatermarkText.length == 0 && ! p.cbAppendFileNames ) {
		return true;
	}
	
	// Create an art layer for the text
	var artLayerRef = d.artLayers.add();
	artLayerRef.kind = LayerKind.TEXT;
	artLayerRef.opacity = parseInt(p.etWaterMarkOpacity);

	// If we need to append the filename, grab it from docFileName
	var finalText = p.etWatermarkText;
	if ( p.cbAppendFileNames ) {
		var str = unescape(docFileName);
		var namePlusExt = str.split('.');
		finalText += namePlusExt[0];
	}
	
	// Grab the text layer from the art layer
	var textItemRef = artLayerRef.textItem;
	textItemRef.contents = finalText;
	textItemRef.font = watermark.getFont( p.ddFontFamily, p.ddFontStyle );
	// convert from px text and px image to font point size here first
	// on second thought, they like it the way it is
	// or better yet they like the odd behavior? go figure
	textItemRef.size = UnitValue(parseInt(p.etTextSize), "px");
	textItemRef.color.rgb.hexValue = p.cpTextColorPicker;
	
	watermark.rasterizeLayer();

	watermark.alignWatermark( d, p );
	
	artLayerRef = null;
	textItemRef = null;	
	
	return true;
}

///////////////////////////////////////////////////////////////////////////////
// Function: alignWatermark
// Usage: Align the newly placed text layer or image according to parameters
///////////////////////////////////////////////////////////////////////////////
watermark.alignWatermark = function( d, p ) {
	
	d.selection.selectAll();

	var horizUnit = watermark.stringToUnitValue( p.etHorizontalOffset, "px" );
	horizUnit.value *= 2;
	horizUnit.baseUnit = UnitValue(d.width.as(horizUnit.type), horizUnit.type);
	var horizonalOffset = Number(100 - horizUnit.as("%"));
	if (horizonalOffset < 1.0 && horizonalOffset > -1.0 ) {
		horizonalOffset = 1.0;
	}
	
	var vertUnit = watermark.stringToUnitValue( p.etVerticalOffset, "px" );
	vertUnit.value *= 2;
	vertUnit.baseUnit = UnitValue(d.height.as(vertUnit.type), vertUnit.type);
	var verticalOffset = Number(100 - vertUnit.as("%"));
	if (verticalOffset < 1.0 && verticalOffset > -1.0 ) {
		verticalOffset = 1.0;
	}

	watermark.translateSelection( horizonalOffset, verticalOffset );

	switch( p.ddPosition ) {
		case localize("$$$/watermark/combobox/ur=Upper Right"):
			var alignTopEdges = charIDToTypeID( "AdTp" );
			watermark.alignEdges( alignTopEdges );
			var alignRightEdges = charIDToTypeID( "AdRg" );
			watermark.alignEdges( alignRightEdges );
			break;
		case localize("$$$/watermark/combobox/ul=Upper Left"):
			var alignTopEdges = charIDToTypeID( "AdTp" );
			watermark.alignEdges( alignTopEdges );
			var alignLeftEdges = charIDToTypeID( "AdLf" );
			watermark.alignEdges( alignLeftEdges );
			break;
		case localize("$$$/watermark/combobox/lr=Lower Right"):
			var alignBottomEdges = charIDToTypeID( "AdBt" );
			watermark.alignEdges( alignBottomEdges );
			var alignRightEdges = charIDToTypeID( "AdRg" );
			watermark.alignEdges( alignRightEdges );
			break;
		case localize("$$$/watermark/combobox/ll=Lower Left"):
			var alignBottomEdges = charIDToTypeID( "AdBt" );
			watermark.alignEdges( alignBottomEdges );
			var alignLeftEdges = charIDToTypeID( "AdLf" );
			watermark.alignEdges( alignLeftEdges );
			break;
		case localize("$$$/watermark/combobox/cb=Center Bottom"):
			var alignBottomEdges = charIDToTypeID( "AdBt" );
			watermark.alignEdges( alignBottomEdges );
			watermark.moveSelection(horizUnit.as("%") / 2.0, 0.0);
			var alignHorizontalCenters = charIDToTypeID( "AdCH" );
			watermark.alignEdges( alignHorizontalCenters );
			break;
		case localize("$$$/watermark/combobox/ct=Center Top"):
			var alignTopEdges = charIDToTypeID( "AdTp" );
			watermark.alignEdges( alignTopEdges );
			watermark.moveSelection(horizUnit.as("%") / 2.0, 0.0);
			var alignHorizontalCenters = charIDToTypeID( "AdCH" );
			watermark.alignEdges( alignHorizontalCenters );
			break;
		case localize("$$$/watermark/combobox/cl=Center Left"):
			var alignLeftEdges = charIDToTypeID( "AdLf" );
			watermark.alignEdges( alignLeftEdges );
			watermark.moveSelection(0.0, vertUnit.as("%") / 2.0);
			var alignVerticalCenters = charIDToTypeID( "AdCV" );
			watermark.alignEdges( alignVerticalCenters );
			break;
		case localize("$$$/watermark/combobox/cr=Center Right"):
			var alignRightEdges = charIDToTypeID( "AdRg" );
			watermark.alignEdges( alignRightEdges );
			watermark.moveSelection(0.0, vertUnit.as("%") / 2.0);
			var alignVerticalCenters = charIDToTypeID( "AdCV" );
			watermark.alignEdges( alignVerticalCenters );
			break;
		default: // and Center goes here
			watermark.moveSelection(horizUnit.as("%") / 2.0, vertUnit.as("%") / 2.0);
			var alignHorizontalCenters = charIDToTypeID( "AdCH" );
			watermark.alignEdges( alignHorizontalCenters );
			var alignVerticalCenters = charIDToTypeID( "AdCV" );
			watermark.alignEdges( alignVerticalCenters );
			break;
	}
	
	d.selection.deselect();

}


///////////////////////////////////////////////////////////////////////////////
// Function: alignEdges
// Usage: An easy way to cheat to get two layers to lign up
// Input: alignment to use
//~ 	gAlignBottomEdges = charIDToTypeID( "AdBt" );
//~ 	gAlignTopEdges = charIDToTypeID( "AdTp" );
//~ 	gAlignLeftEdges = charIDToTypeID( "AdLf" );
//~ 	gAlignHorizontalCenters = charIDToTypeID( "AdCH" );
//~ 	gAlignRightEdges = charIDToTypeID( "AdRg" );

//~ 	gAlignLeft = charIDToTypeID( "Left" );
//~ 	gAlignCenter = charIDToTypeID( "Cntr" );
//~ 	gAlignRight = charIDToTypeID( "Rght" );
//~ 	gAlignVerticalCenters = charIDToTypeID( "AdCV" );
///////////////////////////////////////////////////////////////////////////////
watermark.alignEdges = function( inAlignment ) {
	var id130 = charIDToTypeID( 'Algn' );
    var desc30 = new ActionDescriptor();
    var id131 = charIDToTypeID( 'null' );
        var ref25 = new ActionReference();
        var id132 = charIDToTypeID( 'Lyr ' );
        var id133 = charIDToTypeID( 'Ordn' );
        var id134 = charIDToTypeID( 'Lnkd' );
        ref25.putEnumerated( id132, id133, id134 );
    desc30.putReference( id131, ref25 );
    var id135 = charIDToTypeID( 'Usng' );
    var id136 = charIDToTypeID( 'ADSt' );
    desc30.putEnumerated( id135, id136, inAlignment );
	executeAction( id130, desc30, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: translateSelection
// Usage: Translate the current selection by a percentage
// Input: horzP and vertP is the percentage horizontal and vertical
// Return: <nothing> (Altered selection)
///////////////////////////////////////////////////////////////////////////////
watermark.translateSelection = function( horzP, vertP ) {
	// this gives me the document does not have a selection
	// activeDocument.selection.resizeBoundary(horzP, vertP);
	var idTrnf = charIDToTypeID( "Trnf" );
    var desc55 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref35 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idfsel = charIDToTypeID( "fsel" );
        ref35.putProperty( idChnl, idfsel );
    desc55.putReference( idnull, ref35 );
    var idFTcs = charIDToTypeID( "FTcs" );
    var idQCSt = charIDToTypeID( "QCSt" );
    var idQcsa = charIDToTypeID( "Qcsa" );
    desc55.putEnumerated( idFTcs, idQCSt, idQcsa );
    var idOfst = charIDToTypeID( "Ofst" );
        var desc56 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc56.putUnitDouble( idHrzn, idPxl, 0.000000 );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPxl = charIDToTypeID( "#Pxl" );
        desc56.putUnitDouble( idVrtc, idPxl, 0.000000 );
    var idOfst = charIDToTypeID( "Ofst" );
    desc55.putObject( idOfst, idOfst, desc56 );
    var idWdth = charIDToTypeID( "Wdth" );
    var idPrc = charIDToTypeID( "#Prc" );
    desc55.putUnitDouble( idWdth, idPrc, horzP );
    var idHght = charIDToTypeID( "Hght" );
    var idPrc = charIDToTypeID( "#Prc" );
    desc55.putUnitDouble( idHght, idPrc, vertP );
	executeAction( idTrnf, desc55, DialogModes.NO );

}

///////////////////////////////////////////////////////////////////////////////
// Function: moveSelection
// Usage: Move the current selection by a percentage
// Input: horzP and vertP is the percentage horizontal and vertical
// Return: <nothing> (Altered selection)
///////////////////////////////////////////////////////////////////////////////
watermark.moveSelection = function( horzP, vertP ) {
	var idmove = charIDToTypeID( "move" );
    var desc11 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref7 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idfsel = charIDToTypeID( "fsel" );
        ref7.putProperty( idChnl, idfsel );
    desc11.putReference( idnull, ref7 );
    var idT = charIDToTypeID( "T   " );
        var desc12 = new ActionDescriptor();
        var idHrzn = charIDToTypeID( "Hrzn" );
        var idPrc = charIDToTypeID( "#Prc" );
        desc12.putUnitDouble( idHrzn, idPrc, horzP );
        var idVrtc = charIDToTypeID( "Vrtc" );
        var idPrc = charIDToTypeID( "#Prc" );
        desc12.putUnitDouble( idVrtc, idPrc, vertP );
    var idOfst = charIDToTypeID( "Ofst" );
    desc11.putObject( idT, idOfst, desc12 );
	executeAction( idmove, desc11, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: fitImage
// Usage: copied and modified from FitImage.jsx
// Input: width height and interpolation method
// Return:
///////////////////////////////////////////////////////////////////////////////
watermark.fitImage = function( inWidth, inHeight, inInterpolation ) {
	
	// Gaurd against no text coming in, we also gaurd in the dialog
	if ( inWidth == undefined || inHeight == undefined ) {
		if (watermark.showAlerts) alert( localize("$$$/watermark/alert/whmustdefine=Width and Height must be defined to use FitImage function.") );
		return;
	}

	var docWidth;
	var docHeight;
	var docRatio;
	var newWidth;
	var newHeight;
	var resolution = app.activeDocument.resolution;
	
	// original width, height
	docWidth = (1.0 * app.activeDocument.width * resolution) / 72.0; // decimal inches assuming 72 dpi (used in docRatio)
	docHeight = (1.0 * app.activeDocument.height * resolution) / 72.0; // ditto
	
	if (docWidth < 1.0 || docHeight < 1.0)
		return true; // error
	
	if (inWidth < 1 || inHeight < 1)
		return true; // error
	
	docRatio = docWidth / docHeight; // decimal ratio of original width/height
	
	// NOTE - ccox - 17 Aug 2004 - I added the rounding by 0.5
	// this should solve reported cases of fit image being off by 1 (always under)
	// NOTE - elr - 3 May 2006 - keep original aspect ratio
	// NOTE - mshaw- 16 Sept 2007 commenting out lines and adding lines to match fixed FitImage script
	newWidth = inWidth;
	/*newHeight = ((1.0 * inWidth) / docRatio) + 0.5; // decimal calc
	newHeight = 1 * newHeight; // make integer*/
	newHeight = ((1.0 * inWidth) / docRatio); // decimal calc

	
	if (newHeight > inHeight) {
		/*newWidth = 0.5 + docRatio * inHeight; // decimal calc
		newWidth = 1 * newWidth; // make integer*/
		newWidth = docRatio * inHeight; // decimal calc
		newHeight = inHeight;
	}
	
	if ( typeof( inInterpolation ) == "string" ) {
		var interpoArray = { NearestNeighbor:ResampleMethod.NEARESTNEIGHBOR, Bilinear:ResampleMethod.BILINEAR, Bicubic:ResampleMethod.BICUBIC, BicubicSmoother:ResampleMethod.BICUBICSMOOTHER, BicubicSharper:ResampleMethod.BICUBICSHARPER };
		var lookupstring = inInterpolation.replace(" ", "");
		inInterpolation = interpoArray[ lookupstring ];
	} else {
		alert(localize("$$$/watermark/alert/badinterpolation=Bad interpolation : ") + typeof( inInterpolation ));
	}

    if (app.activeDocument.bitsPerChannel == BitsPerChannelType.THIRTYTWO && 
        (inInterpolation != ResampleMethod.BILINEAR && inInterpolation != ResampleMethod.NEARESTNEIGHBOR)) {
    	inInterpolation = ResampleMethod.BILINEAR;
    }
    
    // resize the image using a good conversion method while keeping the pixel resolution
    // and the aspect ratio the same
    app.activeDocument.resizeImage(newWidth, newHeight, resolution, inInterpolation);

	return false; // no error
}

///////////////////////////////////////////////////////////////////////////////
// Function: unitsToPixels
// Usage: convert a number in the given units to pixels
// Input: value in the current units
//        units of the value to convert
//        resolution of the current value
//        value in pixels for percent conversion 
//        (How exaclty am I supposed to pass that in!)
// Return: value in pixels 
///////////////////////////////////////////////////////////////////////////////
watermark.unitsToPixels = function( value, units, resolution, docValue ) {
    var n = Number( value );
    if ( units.search( "in" ) != -1 ) {
        n = value * resolution;
        return n;
    }
    if ( units.search( "cm" ) != -1 ) {
        n = value / 2.54 * resolution;
        return n;
    }
    if ( units.search( "mm" ) != -1 ) {
        n = value / 25.4 * resolution;
        return n;
    }
    if ( units.search( "points" ) != -1 ) {
        n = value * resolution / 72.72;
        return n;
    }
    if ( units.search( "picas" ) != -1 ) {
        n = value * resolution / 72;
        return n;
    }
    if ( ( units.search( "%" ) != -1 ) && ( docValue != undefined ) ) {
        n = value * docValue / 100.0;
        return n;
    }
    return n;
}

///////////////////////////////////////////////////////////////////////////////
// Function: createUniqueFileName
// Usage: Look at a folder and make sure you don't overwrite a file already
//        present
// Input: inFolder folder as string (contains the trailing / )
//        inFileName name of the file with no folder and no extension
//        inExtension the extension of the file (contains the . to start)
// Return: string for the full path to a file not found in the inFolder
///////////////////////////////////////////////////////////////////////////////
watermark.createUniqueFileName = function( inFolder, inFileName, inExtension ) {
	var originalFileName = inFileName;
	inFileName = inFileName.replace(/[:\/\\*\?\"\<\>\|]/g, "_");  // '/\:*?"<>|' -> '_'
	// if (inFileName != originalFileName) alert(inFileName +":"+ originalFileName);
	var uniqueFileName = inFolder + inFileName + inExtension;
	var fileNumber = 1;
	while ( File( uniqueFileName ).exists ) {
		uniqueFileName = inFolder + inFileName + "_" + fileNumber + inExtension;
		fileNumber++;
	}
	return uniqueFileName;
}

///////////////////////////////////////////////////////////////////////////////
// Function: fileFolderSplitter
// Usage: Given a full path to a file or a file with an extension or no full
//        path and no extension return the .folder, .name and .extension
// Input: string or File, folder seperator is "/" or "\\"
// Return: object containing .folder, .name and .extension
///////////////////////////////////////////////////////////////////////////////
watermark.fileFolderSplitter = function( inFullPath ) {
	this.name = inFullPath;
	this.folder = "";
	this.extension = "";
	if ( inFullPath instanceof File || inFullPath instanceof Folder ) {
		this.name = inFullPath.name;
		this.folder = inFullPath.parent.toString();
	} else {
		if ( typeof this.name == 'string' ) {
			var lastSlash = this.name.lastIndexOf( "/" );
			if ( lastSlash == -1 ) {
				lastSlash = this.name.lastIndexOf( "\\" );
			}
			if ( lastSlash != -1 ) {
				this.folder = this.name.substr( 0, lastSlash );
				this.name = this.name.substr( lastSlash + 1, this.name.length );
			}
		}
	}
	// this.name could have the extension but the folder is ready to go
	// so in theory i don't have to worry about . (dots) in folder names
	if ( typeof this.name == 'string' ) {
		var lastDot = this.name.lastIndexOf( "." );
		if ( lastDot == -1 ) {
			lastDot = this.name.length;
		}
		this.extension = this.name.substr( lastDot + 1, this.name.length );
		this.name = this.name.substr( 0, lastDot );
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: saveAsGIForJPEG
// Usage: Safe a GIF or JPEG file via Save For Web, makes them really small
// Input: file, format, options for each format 'GIFf' "colors" or 'JPEG' "quality"
// Return:
///////////////////////////////////////////////////////////////////////////////
watermark.saveAsGIForJPEG = function( inFileName, inFormat, includeICC, inColorsOrQuality ) {
	try {
	var id124 = charIDToTypeID( "Expr" );
    var desc24 = new ActionDescriptor();
    var id125 = charIDToTypeID( "Usng" );
    var desc25 = new ActionDescriptor();
    var id126 = charIDToTypeID( "Op  " );
    var id127 = charIDToTypeID( "SWOp" );
    var id128 = charIDToTypeID( "OpSa" );
    desc25.putEnumerated( id126, id127, id128 );
    var id129 = charIDToTypeID( "Fmt " );
    var id130 = charIDToTypeID( "IRFm" );
    var id131 = charIDToTypeID( inFormat ); // "GIFf" or "JPEG" goes here
    desc25.putEnumerated( id129, id130, id131 );
    var id132 = charIDToTypeID( "Intr" );
    desc25.putBoolean( id132, inFormat == "GIFf" ); // true for GIFf false for JPEG
	
	if ( inFormat == "GIFf" ) { // only for GIF
		var id133 = charIDToTypeID( "RedA" );
		var id134 = charIDToTypeID( "IRRd" );
		var id135 = charIDToTypeID( "Sltv" );
		desc25.putEnumerated( id133, id134, id135 );
		var id136 = charIDToTypeID( "RChT" );
		desc25.putBoolean( id136, false );
		var id137 = charIDToTypeID( "RChV" );
		desc25.putBoolean( id137, false );
		var id138 = charIDToTypeID( "AuRd" );
		desc25.putBoolean( id138, false );
		var id139 = charIDToTypeID( "NCol" ); // this is the number of colors
		desc25.putInteger( id139, inColorsOrQuality );
		var id140 = charIDToTypeID( "Dthr" );
		var id141 = charIDToTypeID( "IRDt" );
		var id142 = charIDToTypeID( "None" );
		desc25.putEnumerated( id140, id141, id142 );
		var id143 = charIDToTypeID( "DthA" );
		desc25.putInteger( id143, 100 );
		var id144 = charIDToTypeID( "DChS" );
		desc25.putInteger( id144, 0 );
		var id145 = charIDToTypeID( "DCUI" );
		desc25.putInteger( id145, 0 );
		var id146 = charIDToTypeID( "DChT" );
		desc25.putBoolean( id146, false );
		var id147 = charIDToTypeID( "DChV" );
		desc25.putBoolean( id147, false );
		var id148 = charIDToTypeID( "WebS" );
		desc25.putInteger( id148, 0 );
		var id149 = charIDToTypeID( "TDth" );
		var id150 = charIDToTypeID( "IRDt" );
		var id151 = charIDToTypeID( "None" );
		desc25.putEnumerated( id149, id150, id151 );
		var id152 = charIDToTypeID( "TDtA" );
		desc25.putInteger( id152, 100 );
		var id153 = charIDToTypeID( "Loss" );
		desc25.putInteger( id153, 0 );
		var id154 = charIDToTypeID( "LChS" );
		desc25.putInteger( id154, 0 );
		var id155 = charIDToTypeID( "LCUI" );
		desc25.putInteger( id155, 100 );
		var id156 = charIDToTypeID( "LChT" );
		desc25.putBoolean( id156, false );
		var id157 = charIDToTypeID( "LChV" );
		desc25.putBoolean( id157, false );
		var id158 = charIDToTypeID( "Trns" );
		desc25.putBoolean( id158, true );
	} else { // only for JPEG
        var id75 = charIDToTypeID( "Qlty" );
        desc25.putInteger( id75, Math.round( inColorsOrQuality / 12 * 100 ) ); // we use the photoshop 0 to 12, this thing uses 0 to 100
        var id76 = charIDToTypeID( "QChS" );
        desc25.putInteger( id76, 0 );
        var id77 = charIDToTypeID( "QCUI" );
        desc25.putInteger( id77, 0 );
        var id78 = charIDToTypeID( "QChT" );
        desc25.putBoolean( id78, false );
        var id79 = charIDToTypeID( "QChV" );
        desc25.putBoolean( id79, false );
        var id80 = charIDToTypeID( "Optm" );
        desc25.putBoolean( id80, true );
        var id81 = charIDToTypeID( "Pass" );
        desc25.putInteger( id81, 1 );
        var id82 = charIDToTypeID( "blur" );
        desc25.putDouble( id82, 0.000000 );
        var id83 = charIDToTypeID( "EICC" );
        desc25.putBoolean( id83, includeICC );
	} // end only for GIF or JPEG

	var id159 = charIDToTypeID( "Mtt " );
	desc25.putBoolean( id159, true );
	var id160 = charIDToTypeID( "MttR" );
	desc25.putInteger( id160, 255 );
	var id161 = charIDToTypeID( "MttG" );
	desc25.putInteger( id161, 255 );
	var id162 = charIDToTypeID( "MttB" );
	desc25.putInteger( id162, 255 );
	var id163 = charIDToTypeID( "SHTM" );
	desc25.putBoolean( id163, false );
	var id164 = charIDToTypeID( "SImg" );
	desc25.putBoolean( id164, true );
	var id165 = charIDToTypeID( "SSSO" );
	desc25.putBoolean( id165, false );
	var id166 = charIDToTypeID( "SSLt" );
	var list3 = new ActionList();
	desc25.putList( id166, list3 );
	var id167 = charIDToTypeID( "DIDr" );
	desc25.putBoolean( id167, false );
	var id168 = charIDToTypeID( "In  " );
	
	// embed the ICC
	var idEICC = charIDToTypeID( "EICC" );
    desc25.putBoolean( idEICC, true );
	
	// make it sRGB
	var idSWch = charIDToTypeID( "SWch" );
    var idSTch = charIDToTypeID( "STch" );
    var idCHsR = charIDToTypeID( "CHsR" );
    desc25.putEnumerated( idSWch, idSTch, idCHsR );
    
    // add the copyright only
    var idSWmd = charIDToTypeID( "SWmd" );
    var idSTmd = charIDToTypeID( "STmd" );
    // this is for no metadaata var idMDNn = charIDToTypeID( "MDNn" );
    var idMDCp = charIDToTypeID( "MDCp" );
    desc25.putEnumerated( idSWmd, idSTmd, idMDCp );
	
	desc25.putPath( id168, new File( inFileName ) ); // here goes the file name
	var id169 = stringIDToTypeID( "SaveForWeb" );
	desc24.putObject( id125, id169, desc25 );
	executeAction( id124, desc24, DialogModes.NO );
	} catch(e) { if (watermark.showAlerts) alert(localize("$$$/watermark/alert/saveproblem=Save problems: {0} : {1}").replace("{0}", e).replace("{1}", e.line)); }
}

///////////////////////////////////////////////////////////////////////////////
// Function: duplicateLayerToDocument
// Usage: Instead of copy paste use the duplicate layer to new document command
// Input: newLayerName name for the layer in the other document
//        targetDocumentName name for the target document
// Return: <nothing> other document is altered with new layer
///////////////////////////////////////////////////////////////////////////////
watermark.duplicateLayerToDocument = function( newLayerName, targetDocumentName ) {
	var idDplc = charIDToTypeID( "Dplc" );
    var desc54 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref17 = new ActionReference();
        var idLyr = charIDToTypeID( "Lyr " );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref17.putEnumerated( idLyr, idOrdn, idTrgt );
    desc54.putReference( idnull, ref17 );
    var idT = charIDToTypeID( "T   " );
        var ref18 = new ActionReference();
        var idDcmn = charIDToTypeID( "Dcmn" );
        ref18.putName( idDcmn, targetDocumentName );
    desc54.putReference( idT, ref18 );
    var idNm = charIDToTypeID( "Nm  " );
    desc54.putString( idNm, newLayerName );
    var idVrsn = charIDToTypeID( "Vrsn" );
    desc54.putInteger( idVrsn, 2 );
	executeAction( idDplc, desc54, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: mergeVisible
// Usage: Merge all the visible layers in the document, like flatten but not
// Input: <nothing> an active document
// Return: <nothing> a document with some missing layers
///////////////////////////////////////////////////////////////////////////////
watermark.mergeVisible = function() {
	try {
		var idMrgV = charIDToTypeID( "MrgV" );
		executeAction( idMrgV, undefined, DialogModes.NO );
	} catch(e) { ; } // a doc with no layers, no problem
}

///////////////////////////////////////////////////////////////////////////////
// Function: webSharpen
// Usage: Dr. Brown's magical web sharpen thingy
// Input: <nothing> an active document
// Return: <nothing> a document with some nice sharpining
///////////////////////////////////////////////////////////////////////////////
watermark.webSharpen = function( d ) {
// 	d.selection.selectAll();
// 	d.selection.copy();
// 	watermark.newChannel();
// 	d.paste();
// 	watermark.findEdges();
// 	watermark.showCurrentChannel();
// 	watermark.runLevels();
// 	watermark.setSelectionFromChannel();
// 	watermark.deleteCurrentChannel();
	watermark.smartSharpen();
// 	d.selection.deselect();
}

///////////////////////////////////////////////////////////////////////////////
// Function: newChannel
// Usage: Make a new channel and give it some properties
// Input: <nothing> an active document
// Return: <nothing> a new channel on the document
///////////////////////////////////////////////////////////////////////////////
watermark.newChannel = function() {
	var idMk = charIDToTypeID( "Mk  " );
    var desc35 = new ActionDescriptor();
    var idNw = charIDToTypeID( "Nw  " );
        var desc36 = new ActionDescriptor();
        var idNm = charIDToTypeID( "Nm  " );
        desc36.putString( idNm, "Edge Mask" );
        var idClrI = charIDToTypeID( "ClrI" );
        var idMskI = charIDToTypeID( "MskI" );
        var idMskA = charIDToTypeID( "MskA" );
        desc36.putEnumerated( idClrI, idMskI, idMskA );
        var idClr = charIDToTypeID( "Clr " );
            var desc37 = new ActionDescriptor();
            var idRd = charIDToTypeID( "Rd  " );
            desc37.putDouble( idRd, 255.000000 );
            var idGrn = charIDToTypeID( "Grn " );
            desc37.putDouble( idGrn, 0.000000 );
            var idBl = charIDToTypeID( "Bl  " );
            desc37.putDouble( idBl, 0.000000 );
        var idRGBC = charIDToTypeID( "RGBC" );
        desc36.putObject( idClr, idRGBC, desc37 );
        var idOpct = charIDToTypeID( "Opct" );
        desc36.putInteger( idOpct, 50 );
    var idChnl = charIDToTypeID( "Chnl" );
    desc35.putObject( idNw, idChnl, desc36 );
	executeAction( idMk, desc35, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: findEdges
// Usage: Run the find edges command
// Input: <nothing> an active document
// Return: <nothing> I found all the edges
///////////////////////////////////////////////////////////////////////////////
watermark.findEdges = function() {
	var idFndE = charIDToTypeID( "FndE" );
	executeAction( idFndE, undefined, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: showCurrentChannel
// Usage: Show the current channel
// Input: <nothing> an active document
// Return: <nothing> there they are
///////////////////////////////////////////////////////////////////////////////
watermark.showCurrentChannel = function() {
	var idShw = charIDToTypeID( "Shw " );
    var desc66 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var list3 = new ActionList();
            var ref40 = new ActionReference();
            var idChnl = charIDToTypeID( "Chnl" );
            var idOrdn = charIDToTypeID( "Ordn" );
            var idTrgt = charIDToTypeID( "Trgt" );
            ref40.putEnumerated( idChnl, idOrdn, idTrgt );
        list3.putReference( ref40 );
    desc66.putList( idnull, list3 );
	executeAction( idShw, desc66, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: runLevels
// Usage: Run the levels command with our parameters hard coded
// Input: <nothing> an active document
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////
watermark.runLevels = function() {
	var idLvls = charIDToTypeID( "Lvls" );
    var desc67 = new ActionDescriptor();
    var idpresetKind = stringIDToTypeID( "presetKind" );
    var idpresetKindType = stringIDToTypeID( "presetKindType" );
    var idpresetKindCustom = stringIDToTypeID( "presetKindCustom" );
    desc67.putEnumerated( idpresetKind, idpresetKindType, idpresetKindCustom );
    var idAdjs = charIDToTypeID( "Adjs" );
        var list4 = new ActionList();
            var desc68 = new ActionDescriptor();
            var idChnl = charIDToTypeID( "Chnl" );
                var ref41 = new ActionReference();
                var idChnl = charIDToTypeID( "Chnl" );
                var idOrdn = charIDToTypeID( "Ordn" );
                var idTrgt = charIDToTypeID( "Trgt" );
                ref41.putEnumerated( idChnl, idOrdn, idTrgt );
            desc68.putReference( idChnl, ref41 );
            var idInpt = charIDToTypeID( "Inpt" );
                var list5 = new ActionList();
                list5.putInteger( 0 );
                list5.putInteger( 223 );
            desc68.putList( idInpt, list5 );
            var idGmm = charIDToTypeID( "Gmm " );
            desc68.putDouble( idGmm, 2.500000 );
            var idOtpt = charIDToTypeID( "Otpt" );
                var list6 = new ActionList();
                list6.putInteger( 60 );
                list6.putInteger( 255 );
            desc68.putList( idOtpt, list6 );
        var idLvlA = charIDToTypeID( "LvlA" );
        list4.putObject( idLvlA, desc68 );
    desc67.putList( idAdjs, list4 );
	executeAction( idLvls, desc67, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: setSelectionFromChannel
// Usage: Set the selection from the current channel
// Input: <nothing> an active document
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////
watermark.setSelectionFromChannel = function() {
	var idsetd = charIDToTypeID( "setd" );
    var desc69 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref42 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idfsel = charIDToTypeID( "fsel" );
        ref42.putProperty( idChnl, idfsel );
    desc69.putReference( idnull, ref42 );
    var idT = charIDToTypeID( "T   " );
        var ref43 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref43.putEnumerated( idChnl, idOrdn, idTrgt );
    desc69.putReference( idT, ref43 );
	executeAction( idsetd, desc69, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: deleteCurrentChannel
// Usage: Delete the current channel
// Input: <nothing> an active document
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////
watermark.deleteCurrentChannel = function() {
	var idDlt = charIDToTypeID( "Dlt " );
    var desc72 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref46 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref46.putEnumerated( idChnl, idOrdn, idTrgt );
    desc72.putReference( idnull, ref46 );
	executeAction( idDlt, desc72, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: smartSharpen
// Usage: Run the smart sharpen command using last used parameters
// Input: <nothing> an active document
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////
watermark.smartSharpen = function() {
	var idsmartSharpen = stringIDToTypeID( "smartSharpen" );
//    var desc73 = new ActionDescriptor();
//     var idAmnt = charIDToTypeID( "Amnt" );
//     var idPrc = charIDToTypeID( "#Prc" );
//     desc73.putUnitDouble( idAmnt, idPrc, 50.00000);
//     var idRds = charIDToTypeID( "Rds " );
//     var idPxl = charIDToTypeID( "#Pxl" );
//     desc73.putUnitDouble( idRds, idPxl, 0.400000 );
//     var idThsh = charIDToTypeID( "Thsh" );
//     desc73.putInteger( idThsh, 0 );
//     var idAngl = charIDToTypeID( "Angl" );
//     desc73.putInteger( idAngl, 0 );
//     var idmoreAccurate = stringIDToTypeID( "moreAccurate" );
//     desc73.putBoolean( idmoreAccurate, true );
//     var idblur = charIDToTypeID( "blur" );
//     var idblurType = stringIDToTypeID( "blurType" );
//     var idlensBlur = stringIDToTypeID( "lensBlur" );
//     desc73.putEnumerated( idblur, idblurType, idlensBlur );
//     var idpreset = stringIDToTypeID( "preset" );
//     desc73.putString( idpreset, "Default" );
//     var idsdwM = charIDToTypeID( "sdwM" );
//         var desc74 = new ActionDescriptor();
//         var idAmnt = charIDToTypeID( "Amnt" );
//         var idPrc = charIDToTypeID( "#Prc" );
//         desc74.putUnitDouble( idAmnt, idPrc, 0.000000 );
//         var idWdth = charIDToTypeID( "Wdth" );
//         var idPrc = charIDToTypeID( "#Prc" );
//         desc74.putUnitDouble( idWdth, idPrc, 50.000000 );
//         var idRds = charIDToTypeID( "Rds " );
//         desc74.putInteger( idRds, 1 );
//     var idadaptCorrectTones = stringIDToTypeID( "adaptCorrectTones" );
//     desc73.putObject( idsdwM, idadaptCorrectTones, desc74 );
//     var idhglM = charIDToTypeID( "hglM" );
//         var desc75 = new ActionDescriptor();
//         var idAmnt = charIDToTypeID( "Amnt" );
//         var idPrc = charIDToTypeID( "#Prc" );
//         desc75.putUnitDouble( idAmnt, idPrc, 0.000000 );
//         var idWdth = charIDToTypeID( "Wdth" );
//         var idPrc = charIDToTypeID( "#Prc" );
//         desc75.putUnitDouble( idWdth, idPrc, 50.000000 );
//         var idRds = charIDToTypeID( "Rds " );
//         desc75.putInteger( idRds, 1 );
//     var idadaptCorrectTones = stringIDToTypeID( "adaptCorrectTones" );
//     desc73.putObject( idhglM, idadaptCorrectTones, desc75 );
	executeAction( idsmartSharpen, undefined, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: rasterizeLayer
// Usage: Used for text.  Rasterizes the layer.
// Input: <nothing> 
// Return: <nothing>
///////////////////////////////////////////////////////////////////////////////

watermark.rasterizeLayer = function() {
	var idrasterizeLayer = stringIDToTypeID( "rasterizeLayer" );
	var desc34 = new ActionDescriptor();
	var idnull = charIDToTypeID( "null" );
	var ref3 = new ActionReference();
	var idLyr = charIDToTypeID( "Lyr " );
	var idOrdn = charIDToTypeID( "Ordn" );
	var idTrgt = charIDToTypeID( "Trgt" );
	ref3.putEnumerated( idLyr, idOrdn, idTrgt );
	desc34.putReference( idnull, ref3 );
	var idWhat = charIDToTypeID( "What" );
	var idrasterizeItem = stringIDToTypeID( "rasterizeItem" );
	var idType = charIDToTypeID( "Type" );
	desc34.putEnumerated( idWhat, idrasterizeItem, idType );
	executeAction( idrasterizeLayer, desc34, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: openDocs
// Usage: we want the current open documents that have a path if I do 
//        inputFiles = documents then I have a reference and if I add documents
//        it also adds them to my inputFiles I want to copy the current state
// Input: <none>
// Return: array of the current open docs with file references
///////////////////////////////////////////////////////////////////////////////
watermark.openDocs = function() {
	var docs = new Array();
	var i = 0;
	var docIndex = 0;
	var alertNonSavedDocs = false;
	var nonSavedDocs = new Array();
	for ( var i = 0; i < app.documents.length; i++ ) {
		try {
			var temp = app.documents[ i ].fullName;
			docs[ docIndex ] = app.documents[ i ];
			docIndex++;
		}
		catch( e ) {
			if ( e.number == 8103 ) { // this document has not been saved
				alertNonSavedDocs = true;
				nonSavedDocs.push( app.documents[ i ].name );
			} else {
				throw e;
			}
		}
	}
	if ( alertNonSavedDocs ) {
		alert( localize("$$$/watermark/alert/openfileneedsave=Open files must be saved before they can be used by the Watermark Creator.\r The following files will not be processed.\r( {0} )").replace("{0}", nonSavedDocs) );
	}
	return docs;
}

///////////////////////////////////////////////////////////////////////////////
// Function: IsFileOneOfThese
// Usage: Given a file see if the extension is in the list
// Input: inFileName a file with an extension
//        inArrayOfFileExtensions an array of extensions
// Return: true if the file extension is in the list
///////////////////////////////////////////////////////////////////////////////
watermark.isFileOneOfThese = function( inFileName, inArrayOfFileExtensions ) {
	var lastDot = inFileName.toString().lastIndexOf( "." );
	if ( lastDot == -1 ) {
		return false;
	}
	var strLength = inFileName.toString().length;
	var extension = inFileName.toString().substr( lastDot + 1, strLength - lastDot );
	extension = extension.toUpperCase();
	for (var i = 0; i < inArrayOfFileExtensions.length; i++ ) {
		if ( extension == inArrayOfFileExtensions[i] ) {
			return true;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Function: isFileOneOfThese
// Usage: Given a file name and a list of types, only runs on Mac
// Dependant: isMacintoshOS
// Input: inFileName a full path to a file
//        inArrayOfFileTypes an array of types
// Return: true if the file type is in the list
///////////////////////////////////////////////////////////////////////////////
watermark.isFileOneOfTheseTypes = function( inFileName, inArrayOfFileTypes ) {
	if ( ! watermark.isMacintoshOS() ) {
		return false;
		}
	var file = new File (inFileName);
	for (var i = 0; i < inArrayOfFileTypes.length; i++ ) {
		if ( file.type == inArrayOfFileTypes[i] ) {
			return true;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////
// Function: IsMacintoshOS
// Usage: Are we running on the Macintosh OS?
// Input: <none>
// Return: true if on a macintosh
///////////////////////////////////////////////////////////////////////////////
watermark.isMacintoshOS = function() {
	if ( $.os.search(/macintosh/i) != -1 ) {
		return true;
	} else {
		return false;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: stringToUnitValue
// Usage: Convert a string to a UnitValue, "6px" "6 px" et. al.
// Input: inString of the number and units, could just be number
//        inUnits if no units detected or bad units this will be the default
// Return: UnitValue
///////////////////////////////////////////////////////////////////////////////
watermark.stringToUnitValue = function(inString, inUnits) {
	var unitValue = "";
	var unitType = "";
	var findingNumbers = 0; // which includes '+' or '-' characters
	var findingStrings = 1;
	var findingWhat = findingNumbers;
	
	for (var i = 0; i < inString.length; i++) {
		if ( ( findingWhat == findingNumbers ) && 
		     ( ( inString[i] >= "0" && inString[i] <= "9" ) ||
		     ( inString[i] == "." ) || 
		     ( inString[i] == '-' ) ||
		     ( inString[i] == '+' ) ) ) {
			unitValue += inString[i];
		} else {
			findingWhat = findingStrings;
		}
		if (findingWhat == findingStrings && inString[i] != " ") {
			unitType += inString[i];
		}
	}

	// check for a valid unit types, these are from the documentation
	// "centimeters" and "centimeter" is actually valid but come on,
	// nobody would type those into a edit text box
	if (unitType.length == 0 || ( unitType != "px"  && unitType != "in"  &&
	        unitType != "ft" && unitType != "yd"  && unitType != "mi"  && 
	        unitType != "mm" && unitType != "cm"  && unitType != "m"   && 
	        unitType != "km" && unitType != "pt"  && unitType != "pc"  &&
	        unitType != "tpt"&& unitType != "tpc" && unitType != "ci" &&
	        unitType != "%" ) ) {

	    unitType = inUnits;
	}

	return new UnitValue(unitValue, unitType);
}

///////////////////////////////////////////////////////////////////////////////
// Function: mySave
// Usage: Save the doc as either Save For web or Photoshop JPEG
// Input: params, is information from the dialog
//        fullPath, is the full path to the original document
//        doc, the document we are trying to save
// Return: a copy of the document in the correct location
///////////////////////////////////////////////////////////////////////////////
watermark.mySave = function(params, fullPath, doc) {
	try {
		var splitUp = new watermark.fileFolderSplitter( fullPath );
	} catch(e) {
		var splitUp = new Object();
		splitUp.name = "Watermark";
	}
	var uniqueFileName = watermark.createUniqueFileName( params.stSelectedOutput + "/", splitUp.name, ".jpg" ); //  + splitUp.extension );
	var q = Number( params.etQuality );
	if ( params.rbSaveForWeb ) {
		watermark.saveAsGIForJPEG( uniqueFileName, "JPEG", false /* includeiccProfile */, q );
	} else {
		doc.bitsPerChannel = BitsPerChannelType.EIGHT;
		var jpegOptions = new JPEGSaveOptions();
		jpegOptions.quality = q;
		jpegOptions.embedColorProfile = true;
		doc.saveAs( File( uniqueFileName ), jpegOptions, true /* as a copy to get rid of alphas and other stuff */ );
	}
}

///////////////////////////////////////////////////////////////////////////////
// Function: fitOnScreen
// Usage: Fit the current image on screen for best view
// Input: <nothing> an open document
// Return: <nothing> altered view
///////////////////////////////////////////////////////////////////////////////
watermark.fitOnScreen = function() {
	var idslct = charIDToTypeID( "slct" );
    var desc106 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    var ref63 = new ActionReference();
    var idMn = charIDToTypeID( "Mn  " );
    var idMnIt = charIDToTypeID( "MnIt" );
    var idFtOn = charIDToTypeID( "FtOn" );
    ref63.putEnumerated( idMn, idMnIt, idFtOn );
    desc106.putReference( idnull, ref63 );
	executeAction( idslct, desc106, DialogModes.NO );
}

///////////////////////////////////////////////////////////////////////////////
// Function: getFont
// Usage: A little helper that grabs the font given the family and stye
// Input: family (e.g. Arial), style (e.g. regular)
// Return:  TextFont
///////////////////////////////////////////////////////////////////////////////

watermark.getFont = function( family, style ) {
	for( var i=0;i<app.fonts.length; i++ ) {
		if( app.fonts[i].family == family && app.fonts[i].style == style ) { 
			return app.fonts[i].postScriptName;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// The following routines are what transfer data in and out of the swf panel
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// Function: selectFolder
// Usage: Called from the panel so we can select the output folder
// Input: last folder the user selected
// Return: XML describing the full path to the folder
///////////////////////////////////////////////////////////////////////////////
watermark.selectFolder = function(lastFolder) {
	var s = "";
	try {
		var lf = Folder(decodeURIComponent(lastFolder));
		var f = null;
		if (lf.exists) {
			f = lf.selectDlg(localize("$$$/watermark/selectWatermarkFolder=Pick a folder"));
		} else {
			f = Folder.selectDialog(localize("$$$/watermark/selectWatermarkFolder=Pick a folder"));
		}
		if (f != null) {
			s = watermark.stringToXML(encodeURIComponent(f.fsName.toString()));
		}	
	}
	catch(e) { if (watermark.debugAlerts) alert(e + " : " + e.line); }
	return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: selectWatermark
// Usage: Called from the panel so we can select the watermark file
// Input: none
// Return: XML describing the full path to the watermark file
// Note: Use the photoshop open dialog so the file formats are filtered for us
//       vs the File.openDialog or fileObj.openDlg
///////////////////////////////////////////////////////////////////////////////
watermark.selectWatermark = function() {
	var s = "";
	try {
		var f = app.openDialog();
		if (f.length > 0) {
			s = watermark.stringToXML(encodeURIComponent(f[0].fsName.toString()));
		}
	}
	catch(e) { if (watermark.debugAlerts) alert(e + " : " + e.line); }
	return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: getFonts
// Usage: Called from the panel at creation to get the list of fonts.
// Input: <nothing>
// Return: Pseudo XML describing the fonts, including families and styles
///////////////////////////////////////////////////////////////////////////////
watermark.getFonts = function()
{
	//===============================================================
	// Hack Alert: 
	// CSXS Doesn't like sending string's with '<' or '>', so I'm using '{' and '}' instead.
	// This needs to be converted back via AS3's replace() call on the Actionscript side.
	//===============================================================
	
	var lastFontFamily =  app.fonts[0].family;
	var xml =  "<?xml version=\"1.0\" ?>";

	xml += "<fontlist numFonts=\"" + app.fonts.length + "\">";
	xml += "<font family=\"" + app.fonts[0].family + "\">";
	//xml += "{family}" + app.fonts[0].family + "{/family}";
	
	// Loop through each font
	for( var i=0; i<app.fonts.length; i++ )
	{		
		// Check if we have a new font family. If so, add a new xml tag.
		if( lastFontFamily != app.fonts[i].family ) 
		{
			lastFontFamily = app.fonts[i].family;
			xml += "</font>";
			xml += "<font family=\"" + app.fonts[i].family + "\">";
		}
	
		// Add the current style
		xml += "<style>"+ app.fonts[i].style + "</style>";
	}

	xml += "</font>";
	xml += "</fontlist>";

	// Hide the real XML blob as a string in the CSXS object xml we use to pass it back.
	var parentXML = "<object>";
	parentXML += watermark.propertyToXML(escape(xml),"xmlString");
	parentXML += "</object>";

	return parentXML;
}

///////////////////////////////////////////////////////////////////////////////
// Function: getNumFonts
// Usage:  Gets the number of fonts from Photoshop
// Returns: XML object
///////////////////////////////////////////////////////////////////////////////
watermark.getNumFonts = function()
{
	var parentXML = "<object>";
	parentXML += watermark.propertyToXML(app.fonts.length,"numFonts");
	parentXML += "</object>";
	
	return parentXML;
}

///////////////////////////////////////////////////////////////////////////////
// Function: stringToXML
// Usage: Returns to the panel must be wrapped in XML syntax
// Input: inString string to wrap in XML
// Return: string in XML syntax
///////////////////////////////////////////////////////////////////////////////
watermark.stringToXML = function(inString) {
    var s = '<object>';
    s += watermark.propertyToXML(0, "error");
    s += watermark.propertyToXML(inString, "message");
    s += '</object>';
    return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: errorToXML
// Usage: Returns to the panel must be wrapped in XML syntax
// Input: inError is an Error object
// Return: Error in XML syntax
///////////////////////////////////////////////////////////////////////////////
watermark.errorToXML = function(inError) {
    var s = '<object>';
    s += watermark.propertyToXML(inError.number, "error");
    s += watermark.propertyToXML(inError.message, "message");
    s += watermark.propertyToXML(inError.line, "line");
    s += watermark.propertyToXML(inError.fileName, "fileName");
    s += '</object>';
    return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: propertyToXML
// Usage: Returns to the panel must be wrapped in XML syntax, helper routine
// Input: inProperty property string
//        inID property id
// Return: property wrapped in XML syntax for return value to the panel
///////////////////////////////////////////////////////////////////////////////
watermark.propertyToXML = function(inProperty, inID) {
    var t = typeof inProperty;
    var s = '<property id="' + inID + '">';
    switch (t) {
        case "number":
            s += '<number>' + inProperty.toString() + '</number>';
            break;
        case "boolean":
            s += '<' + inProperty.toString() + '/>';
            break;
        case "string":
            s += '<string>' + inProperty.toString() + '</string>';
            break;
        // TODO figure this out
        case "object":
            //s += '<object>';
			//for (var i in inProperty) {
			//	s += propertyToXML(inProperty[i], i.toString());
			//}
			//s += '</object>';
            break;
        case "undefined":
            s += '<string>undefined</string>';
            break;
        default:
            if (watermark.debugAlerts) alert('unknown type not supported: ' + t);
            break;
    }
    s += '</property>';
    return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: getZstring
// Usage: Called from the panel so we can get localized zstring
// Input: zstring string
// Return: XML describing the full path to the folder
///////////////////////////////////////////////////////////////////////////////
watermark.getZstring = function(zstring) {
	var s = '<object>';
	s += watermark.propertyToXML(encodeURIComponent(localize(zstring)), "zstring");
	s += '</object>';
	return s;
}

watermark.myAlert = function(zstring) {
	alert(localize(decodeURIComponent(zstring)));
}

watermark.saveParamsToDisk = function( inParamsAsSource ) {
	try {
	// var params = watermark.paramsToParams( inParamsAsSource );
	var saveFile = null;
	Folder.current = Folder( "~/" );
	if ( File.fs == "Windows" ) {
		saveFile = File.saveDialog( localize( "$$$/JavaScripts/Watermark/PickSave=Pick a Watermark file to save" ),
		                            localize( "$$$/JavaScripts/Watermark/PickWin=Watermark Files: *.wtm" ) );
	} else {
		saveFile = File.saveDialog( localize( "$$$/JavaScripts/Watermark/PickSave=Pick a Watermark file to save" ),
		                            watermark.macFilter );
	}
	if ( saveFile != null ) {
		if ( saveFile.exists && File.fs == "Windows" ) { // mac does this warning for us
			if ( confirm(localize( "$$$/JavaScripts/ImageProcessor/FileAlreadyExists=The file already exists. Do you want to replace?" )) == false) {
				return;
			}
		}
		// saveFile.encoding = "UTF8";
		saveFile.open( "w", "TEXT", "????" );
		// unicode signature, this is UTF16 but will convert to UTF8 "EF BB BF"
		// saveFile.write("\uFEFF"); 
		saveFile.write( inParamsAsSource );
		saveFile.close();
	}
	}
	catch(e) {
		var s = watermark.errorToXML(e);
		alert(s);
	}
}

watermark.loadParamsFromDisk = function() {
	var s = "";
	try {
		// var params = new Object();
		var loadFile = null;
		Folder.current = Folder( "~/" );
		if ( File.fs == "Windows" ) {
			loadFile = File.openDialog( localize( "$$$/JavaScripts/Watermark/PickLoad=Pick a Watermark file to load" ), 
			                            localize( "$$$/JavaScripts/Watermark/PickWin=Watermark Files: *.wtm" ) );
		} else {
			loadFile = File.openDialog( localize( "$$$/JavaScripts/Watermark/PickLoad=Pick a Watermark file to load" ),
			                            watermark.macFilter );
		}
		if ( loadFile != null && loadFile.exists ) {
			loadFile.open( "r" );
			s = '<object>';
    		s += watermark.propertyToXML(0, "error");
    		var inDecoded = loadFile.read(loadFile.length);
    		// alert(inDecoded);
    		var inObject = eval(decodeURIComponent(unescape(inDecoded)));
    		// alert("inObject.etWatermarkText:" + inObject.etWatermarkText);
    		s += watermark.propertyToXML(inObject.rbUseLogoFile, "rbUseLogoFile"); 
    		s += watermark.propertyToXML(encodeURIComponent(inObject.etWatermarkText), "etWatermarkText");
    		s += watermark.propertyToXML(encodeURIComponent(inObject.ddFontFamily), "ddFontFamily");
    		s += watermark.propertyToXML(encodeURIComponent(inObject.ddFontStyle), "ddFontStyle");
    		s += watermark.propertyToXML(encodeURIComponent(inObject.etTextSize), "etTextSize"); 
    		s += watermark.propertyToXML(inObject.cpTextColorPicker, "cpTextColorPicker");
    		s += watermark.propertyToXML(inObject.cbAppendFileNames, "cbAppendFileNames");
    		s += watermark.propertyToXML(encodeURIComponent(inObject.stSelectedWatermark), "stSelectedWatermark");
    		s += watermark.propertyToXML(encodeURIComponent(inObject.stSelectedInput), "stSelectedInput");
    		s += watermark.propertyToXML(encodeURIComponent(inObject.stSelectedOutput), "stSelectedOutput");
    		s += watermark.propertyToXML(inObject.cbUseOpenImages, "cbUseOpenImages");
    		s += watermark.propertyToXML(inObject.ddPosition, "ddPosition");
    		s += watermark.propertyToXML(inObject.etWaterMarkSize, "etWaterMarkSize");
    		s += watermark.propertyToXML(inObject.etWaterMarkOpacity, "etWaterMarkOpacity");
    		s += watermark.propertyToXML(inObject.etVerticalOffset, "etVerticalOffset");
    		s += watermark.propertyToXML(inObject.etHorizontalOffset, "etHorizontalOffset");
    		s += watermark.propertyToXML(inObject.etQuality, "etQuality");
    		s += watermark.propertyToXML(inObject.etResolution, "etResolution");
    		s += watermark.propertyToXML(inObject.cbSizeToFit, "cbSizeToFit");
    		s += watermark.propertyToXML(inObject.etSizeToFitW, "etSizeToFitW");
    		s += watermark.propertyToXML(inObject.etSizeToFitH, "etSizeToFitH");
    		s += watermark.propertyToXML(inObject.rbSaveForWeb, "rbSaveForWeb");
    		s += watermark.propertyToXML(inObject.cbSmartSharpen, "cbSmartSharpen");
			s += '</object>';
			loadFile.close();
			// alert(s);
		} else { // this isn't even a file or user canceled
			s = '<object>';
   			s += watermark.propertyToXML(2, "error");
			s += '</object>';
			if (watermark.debugAlerts) {
				alert(s);
			}
		}
	} 
	catch(e) {
		s = watermark.errorToXML(e);
		if (watermark.debugAlerts) {
			alert(s);
		}
	}
	return s;
}

///////////////////////////////////////////////////////////////////////////////
// Function: ReadHeader
// Usage: a very crude xml parser, this reads the "Tag" of the <Tag>Data</Tag>
// Input:
// Return:
///////////////////////////////////////////////////////////////////////////////
watermark.readHeader = function( inFile ) {
	var returnValue = "";
	if ( ! inFile.eof ) {
		var c = "";
		while ( c != "<" && ! inFile.eof ) {
			c = inFile.read( 1 );
		}
		while ( c != ">" && ! inFile.eof ) {
			c = inFile.read( 1 );
			if ( c != ">" ) {
				returnValue += c;
			}
		}
	} else {
		returnValue = "end of file";
	}
	return returnValue;
}

///////////////////////////////////////////////////////////////////////////////
// Function: ReadData
// Usage: very crude xml parser, this reads the "Data" of the <Tag>Data</Tag>
// Input:
// Return:
///////////////////////////////////////////////////////////////////////////////
watermark.readData = function( inFile ) {
	var returnValue = "";
	if ( ! inFile.eof ) {
		var c = "";
		while ( c != "<" && ! inFile.eof ) {
			c = inFile.read( 1 );
			if ( c != "<" ) {
				returnValue += c;
			}
		}
		inFile.seek( -1, 1 );
	}
	return returnValue;
}

/////////////////////////////////////////////////////////////////////
// Function: MacXMLFilter
// Input: f, file or folder to check
// Return: true or false, true if file or folder is to be displayed
/////////////////////////////////////////////////////////////////////
watermark.macFilter = function( f ) {
	var wtmExtension = ".wtm";
	var lCaseName = f.name;
	lCaseName.toLowerCase();
	if ( lCaseName.indexOf( wtmExtension ) == f.name.length - wtmExtension.length )
		return true;
	else if ( f.type == 'TEXT' )
		return true;
	else if ( f instanceof Folder )
		return true;
	else
		return false;
}

/////////////////////////////////////////////////////////////////////
// Function: Help
// Input: <none>
// Return: <none> but hopefully a web browser got launched
/////////////////////////////////////////////////////////////////////
watermark.help = function( ) {
	var s = "";
	try {
		
		s = "<HTML><HEAD><META HTTP-EQUIV=REFRESH CONTENT=";
		s += '"0; URL=';
		s += localize("$$$/watermark/url/helpmovie=http://av.adobe.com/russellbrown/watermark_help.mov");
		s += ' ">';
		s += "</HEAD><BODY>";
		s += "<!---CENTER><BR><BR><b><i><span style='font-family:Arial'>";
		s += localize("$$$/watermark/str/redirect=redirecting to documentation...");
		s += "</span></i></b></center--->";
		s += "</BODY></HTML>";
		
		var helpFile = new File( Folder.temp.toString() + '/Watermark.html' );
		if (helpFile.open('w')) {
			helpFile.write( s );
			helpFile.close();
			helpFile.execute();
			s = watermark.stringToXML("Success");
		}
	}
	catch(e) { 
		if (watermark.showAlerts) alert(e + " : " + e.line); 
	}
	return s;
}


// i didn't indent cuz that would be annoying
} // end overall try catch block
catch(e) {
	if (watermark.showAlerts) {
		alert(localize("$$$/watermark/alert/alert_key_value={0} : {1}").replace("{0}", e).replace("{1}", e.line));
	}
}

// end Watermark.jsx
